/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
[ProgCnet - Boum!]
Auteur: Thaeron
Projet: "serveur" Boum! (bomberman) du concours ProgCnet
Licence: GPL

Copyright (C) 2007 Thaeron

This program is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation; either version 2 of the 
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 
USA
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef WIN32
#include "apis-win32.h"
#include <windows.h>
#include <winbase.h>
#define ptr_mod_t HMODULE
#define dlopen(x, RTLD_NOW) LoadLibrary(x)
#define dlsym GetProcAddress
#define dlclose FreeLibrary
#define GUI_FILENAME "gui/boum-gui.dll"

#else
#define ptr_mod_t int *
#include <dlfcn.h>
#include <signal.h>
#include <setjmp.h>
#define GUI_FILENAME "gui/boum-gui.so"
#endif

#include "hide_fct.h"

#define NB_TOURS_BOMBE 4
#define PUISSANCE_SUPP 2
#define BONUS_CHANCE 6

#define NB_MOUVEMENT_DEPART 3
#define NB_BOMBE_DEPART 1
#define PUISSANCE_DEPART 6

#define LIMITE_TEMPS 10000

enum
{ JOUEUR = 0, RIEN =
    100, MUR, BRIQUE, BRIQUE_FRAGILE, MOTION_BONUS, BOMBE_BONUS, FLAMME_BONUS, BONUS_FRAGILE, FLAMME, BOMBE,
  BOMBE_JOUEUR
};

enum
{ MORT, VIVANT };

enum
{ PAS_DE_JOUEUR = -11, PHASE_INIT, JOUEUR_MORT, JOUEUR_VIVANT, PLUS_DE_MOUVEMENT, PAS_DE_CHEMIN, ERREUR_TERRAIN, 
ERREUR_COORD, MOUVEMENT_IMPOSSIBLE, PAS_DE_BOMBE, DEJA_UNE_BOMBE, NP };


typedef struct _player_pos_t player_pos_t;
struct _player_pos_t
{
  int x;
  int y;
  unsigned char use;
};

typedef struct _map_t map_t;
struct _map_t
{
  int **map;
  int longueur;
  int hauteur;
  int nb_pos;
  player_pos_t **tab_pos;

  int nb_bombe;
  int current_player;
  int nb_motion_bonus;
  int nb_bombe_bonus;
  int nb_flamme_bonus;
  int tour;
  int date;
  int init;
  int nb_joueur_tot;
  int nb_joueur_lice;
};

typedef struct _ia_t ia_t;
struct _ia_t
{
  char *fichier;
  ptr_mod_t hd;
  void (*fct_turn) (void);
  void (*init_ia) (void);
 #ifdef WIN32
  void (*init_ptr_fonctions) (ptr_fcts_t *);
 #endif
  int x;
  int y;
  int num;
  int puissance;
  int bombe_tot;
  int bombe_dispo;
  int mvt_tot;
  int mvt_dispo;
  int status;
};

typedef struct _gui_module_t gui_module_t;
struct _gui_module_t
{
  ptr_mod_t hd;
  void (*gui_turn) (void);
  void (*gui_init) (void);
  void (*gui_quit) (void);
 #ifdef WIN32
  void (*init_ptr_fonctions) (ptr_fcts_t *);
 #endif
};

typedef struct _bombe_t bombe_t;
struct _bombe_t
{
  int x;
  int y;
  ia_t *player;
  int a_explose;
  int tours;
  int puissance;
};

#ifndef WIN32
sigjmp_buf envx;
#endif

int ia_is_crashed;

int NB_TOUR;
int enable_gui;
gui_module_t *gui_m;

map_t *struct_map;
ia_t **tab_players;

bombe_t **tab_bombe;

/* fonction de apis.c uniquement pour windoz */
#ifdef WIN32
void init_apis_win32 (void);
#endif

/* fonctions de bombe.c */
void ajouter_bombe (void);
bombe_t *get_bombe (int x, int y);
void clean_bombe_tab (void);
void free_bombe (bombe_t ** t, int n);

/* fonctions de boum.c */
int fonction_principoile (void);
void signal_handler (int sigid);

/* fonctions de explosion.c */
int gerer_explosion (int i, int j);
void ajouter_explosion (int k);
void virer_explosion (void);

/* fonctions de gui.c */
gui_module_t *load_gui (char *filename);
void free_gui (gui_module_t * ptr);

/* fonctions de ia.c */
void free_ia (ia_t ** p, int n);
ia_t **load_ia (char *files[], int nb, map_t * sm);

/* fonctions de map.c */
int conv_car_file (char a);
void clean_map (void);
map_t *load_map (char *f_map);
void free_map (map_t * p);
void free_map_pos (map_t * p);

/* fonctions de pathfinding.c */
int **pathfinding_conv_map (void);
void pathfinding_back (int **n_map, int xd, int yd, int xf, int yf);
int boum_pathfinding (int xf, int yf);
int boum_distance_back (int xd, int yd, int xf, int yf);

/* fonctions de routines.c */
char conv_car_map (int a);
void print_map (int **m, int hauteur, int largeur);
void mv_on_map_back (int tx, int ty);
int back_aller (int tx, int ty);
void player_expulse (int id);
