/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
[ProgCnet - Boum!]
Auteur: Thaeron
Projet: "serveur" Boum! (bomberman) du concours ProgCnet
Licence: GPL

Copyright (C) 2007 Thaeron

This program is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation; either version 2 of the 
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 
USA
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#include "boum.h"

void
free_ia (ia_t ** p, int n)
{
  int i;
  for (i = 0; i < n; i++)
    {
      dlclose (p[i]->hd);
      free (p[i]);
    }
  free (p);
}

ia_t **
load_ia (char *files[], int nb, map_t * sm)
{
  int i, ran_nb, pos_dispo;
  ia_t **p;
  if (!nb)
    return (NULL);

  p = (ia_t **) malloc (sizeof (ia_t *) * nb);

  pos_dispo = sm->nb_pos;

  for (i = 0; i < nb; i++)
    {
      p[i] = (ia_t *) malloc (sizeof (ia_t));
      p[i]->hd = dlopen (files[i], RTLD_NOW);
      if (p[i]->hd == NULL)
	{
	#ifndef WIN32
	  fprintf (stderr, "Erreur : impossible de charger l'IA %s %s\n", files[i], dlerror());
	#else
	  fprintf (stderr, "Erreur : impossible de charger l'IA %s\n", files[i]);
	#endif
	  free (p[i]);
	  free_ia (p, i);
	  return (NULL);
	}

      p[i]->init_ia = (void *) dlsym (p[i]->hd, "init_ia");
      if (!p[i]->init_ia)
	{
	  fprintf (stderr, "Erreur : impossible de charger la fonction init_ia dans l'IA %s\n", files[i]);
	  free_ia (p, i + 1);
	  return (NULL);
	}

      p[i]->fct_turn = (void *) dlsym (p[i]->hd, "tour");
      if (!p[i]->fct_turn)
	{
	  fprintf (stderr, "Erreur : impossible de charger la fonction tour dans l'IA %s\n", files[i]);
	  free_ia (p, i + 1);
	  return (NULL);
	}

    #ifdef WIN32
       p[i]->init_ptr_fonctions = (void *) dlsym (p[i]->hd, "init_ptr_fonctions");
      if (!p[i]->init_ptr_fonctions)
	{
	  fprintf (stderr, "Erreur : impossible de charger la fonction d'initialisation des pointeurs de fonctions dans l'IA %s\n", files[i]);
	  free_ia (p, i + 1);
	  return (NULL);
	}
    #endif

      p[i]->bombe_dispo = NB_BOMBE_DEPART;
      p[i]->bombe_tot = NB_BOMBE_DEPART;
      p[i]->puissance = PUISSANCE_DEPART;
      p[i]->mvt_dispo = NB_MOUVEMENT_DEPART;
      p[i]->mvt_tot = NB_MOUVEMENT_DEPART;
      
      p[i]->num = i;
      p[i]->status = VIVANT;
      
      if (!pos_dispo)
	{
	  fprintf (stderr, "Erreur : plus de position disponible sur la map\n");
	  free_ia (p, i + 1);
	  return (NULL);
	}

      do
	{
	  ran_nb = rand () % sm->nb_pos;
	}
      while (sm->tab_pos[ran_nb]->use);

      sm->tab_pos[ran_nb]->use = 1;
      p[i]->x = sm->tab_pos[ran_nb]->x;
      p[i]->y = sm->tab_pos[ran_nb]->y;
      sm->map[p[i]->x][p[i]->y] = JOUEUR + i;
      pos_dispo--;
  /*    printf ("pos %d (%d, %d)\n", ran_nb, p[i]->x, p[i]->y);*/
    }
  sm->nb_joueur_tot = nb;
  sm->nb_joueur_lice = nb;

  return (p);
}
